﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AppLifeCycleDemo
{
    /// <summary>
    /// This static class is responsible for managing the application's event trace
    /// </summary>
    static class EventTracer
    {
        private static StringBuilder _eventTrace = new StringBuilder();

        /// <summary>
        /// Initializes the trace
        /// </summary>
        public static string EventTrace
        {
            get { return _eventTrace.ToString(); }
        }

        /// <summary>
        /// Sets the hosting application object
        /// </summary>
        public static App HostingApp { get; set; }

        /// <summary>
        /// SUbscribe to this event to get trace messages
        /// </summary>
        public static event EventHandler<EventLoggedArgs> TraceAdded;

        /// <summary>
        /// Clears trace messages
        /// </summary>
        public static void Reset()
        {
            _eventTrace.Clear();
        }

        /// <summary>
        /// Sets the trace messages to the specified string
        /// </summary>
        /// <param name="message">Trace message to set</param>
        public static void SetEventTrace(string message)
        {
            _eventTrace.Clear();
            _eventTrace.Append(message);
        }

        /// <summary>
        /// Writes a formatted message to the event trace
        /// </summary>
        /// <param name="message">Message string with placeholders</param>
        /// <param name="messageParams">Message parameters</param>
        public static void Write(string message, params object[] messageParams)
        {
            var formattedMessage = String.Format(message, messageParams);
            var newMessage = String.Format("{0}: {1}", DateTime.Now, formattedMessage);
            _eventTrace.Append(newMessage);
            if (TraceAdded != null)
            {
                TraceAdded(HostingApp, new EventLoggedArgs { Message = newMessage });
            }
        }

        /// <summary>
        /// Writes a formatted message to the event trace and closes with a new line
        /// </summary>
        /// <param name="message">Message string with placeholders</param>
        /// <param name="messageParams">Message parameters</param>
        public static void WriteLine(string message, params object[] messageParams)
        {
            Write(message + "\n", messageParams);
        }
    }

    /// <summary>
    /// Arguments of the TraceAdded event
    /// </summary>
    class EventLoggedArgs : EventArgs
    {
        public string Message { get; set; }
    }
}
