﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.ApplicationSettings;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Popups;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace SimplePuzzle
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private GameController _game;
        private int _height;
        private int _width;

        public MainPage()
        {
            this.InitializeComponent();
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            _game = new GameController(PuzzleCanvas);
            _game.GameStarted += (s, arg) => VisualStateManager.GoToState(this, "InProgress", true);

            _game.ShuffleStarted += (s, arg) =>
                    ResetGameButton.IsEnabled = false;
            _game.ShuffleCompleted += (s, arg) =>
                    ResetGameButton.IsEnabled = true;

            _game.GameCompleted += GameCompleted;
            _height = 3;
            _width = 3;
            StartNewGame();
        }

        private void StartNewGame()
        {
            _game.StartNew(_height, _width);
        }

        private async void GameCompleted(object sender, EventArgs e)
        {
            VisualStateManager.GoToState(this, "Completed", true);
            var messageDialog = new MessageDialog(
                "Sie haben das Puzzle erfolgreich gelöst!",
                "Spiel beendet");
            messageDialog.Commands.Add(
                new UICommand("OK"));
            await messageDialog.ShowAsync();
            _game.StartNew(_height, _width);
        }

        private async void NewGameButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            if (_game.State == GameState.InProgress ||
                _game.State == GameState.Shuffle)
            {
                bool abort = true;
                var messageDialog = new MessageDialog(
                    "Sind Sie sicher, dass Sie dieses Spiel abbrechen " +
                    "und ein neues starten wollen?",
                    "Das Spiel läuft");
                messageDialog.Commands.Add(
                    new UICommand("Ja", (command) => { abort = true; }));
                messageDialog.Commands.Add(
                    new UICommand("Nein", (command) => { abort = false; }));
                messageDialog.DefaultCommandIndex = 1;
                await messageDialog.ShowAsync();
                if (!abort) return;
            }
            _game.StartNew(_height, _width);
            CommandBar.IsOpen = false;
        }

        private async void ResetGameButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            if (_game.State == GameState.InProgress)
            {
                bool restart = true;
                var messageDialog = new MessageDialog(
                    "Sind Sie sicher, dass Sie dieses Spiel neu " +
                    "starten wollen?",
                    "Das Spiel läuft ");
                messageDialog.Commands.Add(
                    new UICommand("Ja", (command) => { restart = true; }));
                messageDialog.Commands.Add(
                    new UICommand("Nein", (command) =>
                    { restart = false; }));
                messageDialog.DefaultCommandIndex = 0;
                messageDialog.CancelCommandIndex = 1;
                await messageDialog.ShowAsync();
                if (!restart) return;
            }
            _game.RestoreStartPositions();
            CommandBar.IsOpen = false;
        }
    }
}
