﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Popups;
using Windows.UI.ApplicationSettings;
using Windows.Storage;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace SimplePuzzle
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private const string SIZE3BY3 = "size3by3";
        private const string SIZE3BY4 = "size3by4";
        private const string SIZE4BY4 = "size4by4";

        private GameController _game;
        private int _height;
        private int _width;

        public MainPage()
        {
            this.InitializeComponent();
        }

        private async void Page_Loaded(object sender, RoutedEventArgs e)
        {
            SettingsPane.GetForCurrentView().CommandsRequested +=
                SettingsCommandsRequested;
            _game = new GameController(PuzzleCanvas);
            App.Game = _game;
            _game.GameStarted += (s, arg) => VisualStateManager.GoToState(this, "InProgress", true);
            _game.ShuffleStarted += (s, arg) => ResetGameButton.IsEnabled = false;
            _game.ShuffleCompleted += (s, arg) => ResetGameButton.IsEnabled = true;
            _game.GameCompleted += GameCompleted;
            _height = 3;
            _width = 3;
            bool resume = false;
            if (App.IsStateSaved())
            {
                var messageDialog = new MessageDialog(
                    "Möchten Sie ein neues Spiel starten oder das " +
                    "gespeichert fortsetzen?",
                    "Fortsetzen");
                messageDialog.Commands.Add(
                    new UICommand("Neues Spiel", (command) => { resume = false; }));
                messageDialog.Commands.Add(
                    new UICommand("Fortsetzen", (command) => { resume = true; }));
                messageDialog.DefaultCommandIndex = 1;
                await messageDialog.ShowAsync();
            }
            if (resume) App.RestorePuzzleState();
            else StartNewGame();
        }

        private void StartNewGame()
        {
            _game.StartNew(_height, _width);
        }

        private async void GameCompleted(object sender, EventArgs e)
        {
            VisualStateManager.GoToState(this, "Completed", true);
            var messageDialog = new MessageDialog(
                "Sie haben das Puzzle erfolgreich gelöst!",
                "Spiel beendet");
            messageDialog.Commands.Add(
                new UICommand("OK"));
            App.RemovePuzzleState();
            await messageDialog.ShowAsync();
            _game.StartNew(_height, _width);
        }

        private async void NewGameButtonTapped(object sender, Windows.UI.Xaml.Input.TappedRoutedEventArgs e)
        {
            if (_game.State == GameState.InProgress || _game.State == GameState.Shuffle)
            {
                bool abort = true;
                var messageDialog = new MessageDialog(
                    "Sind Sie sicher, dass Sie dieses Spiel abbrechen " +
                    "und ein neues starten wollen?",
                    "Das Spiel läuft");
                messageDialog.Commands.Add(
                    new UICommand("Ja", (command) => { abort = true; }));
                messageDialog.Commands.Add(
                    new UICommand("Nein", (command) => { abort = false; }));
                messageDialog.DefaultCommandIndex = 1;
                await messageDialog.ShowAsync();
                if (!abort) return;
            }
            _game.StartNew(_height, _width);
            CommandBar.IsOpen = false;
        }

        private async void ResetGameButtonTapped(object sender, Windows.UI.Xaml.Input.TappedRoutedEventArgs e)
        {
            if (_game.State == GameState.InProgress)
            {
                bool restart = true;
                var messageDialog = new MessageDialog(
                    "Sind Sie sicher, dass Sie dieses Spiel neu " +
                    "starten wollen?",
                    "Das Spiel läuft ");
                messageDialog.Commands.Add(
                    new UICommand("Ja", (command) => { restart = true; }));
                messageDialog.Commands.Add(
                    new UICommand("Nein", (command) => { restart = false; }));
                messageDialog.DefaultCommandIndex = 0;
                messageDialog.CancelCommandIndex = 1;
                await messageDialog.ShowAsync();
                if (!restart) return;
            }
            _game.RestoreStartPositions();
            CommandBar.IsOpen = false;
        }

        void SettingsCommandsRequested(SettingsPane sender, SettingsPaneCommandsRequestedEventArgs args)
        {
            var size3By3 = new SettingsCommand(SIZE3BY3, "3x3", OnSettingsCommand);
            args.Request.ApplicationCommands.Add(size3By3);
            var size3By4 = new SettingsCommand(SIZE3BY4, "3x4", OnSettingsCommand);
            args.Request.ApplicationCommands.Add(size3By4);
            var size4By4 = new SettingsCommand(SIZE4BY4, "4x4", OnSettingsCommand);
            args.Request.ApplicationCommands.Add(size4By4);
        }

        void OnSettingsCommand(IUICommand command)
        {
            var id = command.Id.ToString();
            _height = id == SIZE4BY4 ? 4 : 3;
            _width = id == SIZE3BY3 ? 3 : 4;
        }
    }
}