﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Notifications;
using System.Xml.Linq;
using Windows.Data.Xml.Dom;
using Windows.Globalization;
using Windows.Globalization.DateTimeFormatting;
using Windows.UI.StartScreen;
using Windows.ApplicationModel.Activation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace LiveTiles
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private const string SECONDARY_TILE_ID = "SecondaryTile";

        internal static MainPage Current { get; private set; }

        public MainPage()
        {
            this.InitializeComponent();
            Current = this;
            FeatureList.SelectedIndex = 0;
        }

        private void ResetAll()
        {
            Feature1Input.Visibility = Visibility.Collapsed;
            Feature1Output.Visibility = Visibility.Collapsed;
            Feature2Input.Visibility = Visibility.Collapsed;
            Feature2Output.Visibility = Visibility.Collapsed;
            Feature3Input.Visibility = Visibility.Collapsed;
            Feature3Output.Visibility = Visibility.Collapsed;
            Feature4Input.Visibility = Visibility.Collapsed;
            Feature4Output.Visibility = Visibility.Collapsed;
            Feature5Input.Visibility = Visibility.Collapsed;
            Feature5Output.Visibility = Visibility.Collapsed;
            Feature6Input.Visibility = Visibility.Collapsed;
            Feature6Output.Visibility = Visibility.Collapsed;
            Feature7Input.Visibility = Visibility.Collapsed;
            Feature7Output.Visibility = Visibility.Collapsed;
        }

        private void FeatureSelected(object sender, SelectionChangedEventArgs e)
        {
            ResetAll();
            switch (FeatureList.SelectedIndex)
            {
                case 0:
                    Feature1Input.Visibility = Visibility.Visible;
                    Feature1Output.Visibility = Visibility.Visible;
                    break;
                case 1:
                    Feature2Input.Visibility = Visibility.Visible;
                    Feature2Output.Visibility = Visibility.Visible;
                    break;
                case 2:
                    Feature3Input.Visibility = Visibility.Visible;
                    Feature3Output.Visibility = Visibility.Visible;
                    break;
                case 3:
                    Feature4Input.Visibility = Visibility.Visible;
                    Feature4Output.Visibility = Visibility.Visible;
                    break;
                case 4:
                    Feature5Input.Visibility = Visibility.Visible;
                    Feature5Output.Visibility = Visibility.Visible;
                    break;
                case 5:
                    Feature6Input.Visibility = Visibility.Visible;
                    Feature6Output.Visibility = Visibility.Visible;
                    break;
                case 6:
                    Feature7Input.Visibility = Visibility.Visible;
                    Feature7Output.Visibility = Visibility.Visible;
                    break;
            }
        }

        private void SendTextButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            var bindingSquare =
                new XElement("binding",
                    new XAttribute("template", "TileSquareText04"),
                    new XElement("text",
                        new XAttribute("id", "1"), Message.Text));
            var bindingWide =
                new XElement("binding",
                    new XAttribute("template", "TileWideText04"),
                    new XElement("text",
                        new XAttribute("id", "1"), Message.Text));
            var notificationX =
                new XElement("tile",
                    new XElement("visual", bindingSquare, bindingWide));

            var tileXml = new XmlDocument();
            tileXml.LoadXml(notificationX.ToString());
            TileUpdateManager.CreateTileUpdaterForApplication().Update(new TileNotification(tileXml));
            Feature1OutputText.Text = notificationX.ToString();
        }

        private void ClearNotificationsButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            TileUpdateManager.CreateTileUpdaterForApplication().Clear();
            Feature5OutputText.Text = "All tile notifications cleared.";
        }

        private void SendNumberButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            var notificationX =
                new XElement("badge",
                    new XAttribute("value", 34));

            var tileXml = new XmlDocument();
            tileXml.LoadXml(notificationX.ToString());
            BadgeUpdateManager.CreateBadgeUpdaterForApplication().Update(new BadgeNotification(tileXml));
            Feature2OutputText.Text = notificationX.ToString();
        }

        private void SendGlyph1ButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            var notificationX =
                new XElement("badge",
                    new XAttribute("value", "newMessage"));

            var tileXml = new XmlDocument();
            tileXml.LoadXml(notificationX.ToString());
            BadgeUpdateManager.CreateBadgeUpdaterForApplication().Update(new BadgeNotification(tileXml));
            Feature2OutputText.Text = notificationX.ToString();
        }

        private void SendGlyph2ButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            var notificationX =
                new XElement("badge",
                    new XAttribute("value", "playing"));

            var tileXml = new XmlDocument();
            tileXml.LoadXml(notificationX.ToString());
            BadgeUpdateManager.CreateBadgeUpdaterForApplication().Update(new BadgeNotification(tileXml));
            Feature2OutputText.Text = notificationX.ToString();
        }

        private void ClearBadgesButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            BadgeUpdateManager.CreateBadgeUpdaterForApplication().Clear();
        }

        private void SendAppImageButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            var bindingSquare =
                new XElement("binding",
                    new XAttribute("template", "TileSquareImage"),
                    new XElement("image",
                        new XAttribute("id", "1"),
                        new XAttribute("src", "ms-appx:///Images/SquareImage.png")));
            var bindingWide =
                new XElement("binding",
                    new XAttribute("template", "TileWideText04"),
                    new XElement("text",
                        new XAttribute("id", "1"), "This wide tile has no image."));
            var notificationX =
                new XElement("tile",
                    new XElement("visual", bindingSquare, bindingWide));

            var tileXml = new XmlDocument();
            tileXml.LoadXml(notificationX.ToString());
            TileUpdateManager.CreateTileUpdaterForApplication().Update(new TileNotification(tileXml));
            Feature3OutputText.Text = notificationX.ToString();
        }

        private void SendWebImageButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            var bindingSquare =
                new XElement("binding",
                    new XAttribute("template", "TileSquareImage"),
                    new XElement("image",
                        new XAttribute("id", "1"),
                        new XAttribute("src", "http://i.msdn.microsoft.com/ms348103.NEW_LOGO_VS_LG(en-us,MSDN.10).png")));
            var bindingWide =
                new XElement("binding",
                    new XAttribute("template", "TileWideText04"),
                    new XElement("text",
                        new XAttribute("id", "1"), "This wide tile has no image."));
            var notificationX =
                new XElement("tile",
                    new XElement("visual", bindingSquare, bindingWide));

            var tileXml = new XmlDocument();
            tileXml.LoadXml(notificationX.ToString());
            TileUpdateManager.CreateTileUpdaterForApplication().Update(new TileNotification(tileXml));
            Feature3OutputText.Text = notificationX.ToString();
        }

        private void SendWithExpirationButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            var calendar = new Calendar();
            calendar.SetToNow();
            calendar.AddSeconds(15);
            var timeFormat = new DateTimeFormatter("longtime");
            var expirationTime = calendar.GetDateTime();

            var message = "Tile expires at " + timeFormat.Format(expirationTime);
            var bindingSquare =
                new XElement("binding",
                    new XAttribute("template", "TileSquareText04"),
                    new XElement("text",
                        new XAttribute("id", "1"), message));
            var bindingWide =
                new XElement("binding",
                    new XAttribute("template", "TileWideText04"),
                    new XElement("text",
                        new XAttribute("id", "1"), message));
            var notificationX =
                new XElement("tile",
                    new XElement("visual", bindingSquare, bindingWide));

            var tileXml = new XmlDocument();
            tileXml.LoadXml(notificationX.ToString());
            var tileNotification = new TileNotification(tileXml);
            tileNotification.ExpirationTime = expirationTime;
            TileUpdateManager.CreateTileUpdaterForApplication().Update(tileNotification);
            Feature4OutputText.Text = notificationX.ToString();
        }

        private async void PinTileButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            Uri logo = new Uri("ms-appx:///Assets/Logo2.png");
            Uri smallLogo = new Uri("ms-appx:///Assets/SmallLogo.png");

            var tileActivationArguments = SECONDARY_TILE_ID + " WasPinnedAt=" 
                + DateTime.Now.ToLocalTime().ToString();

            var secondaryTile = new SecondaryTile(SECONDARY_TILE_ID,
                "This is a secondary tile",
                "Name of the tile the user sees when searching for the tile",
                tileActivationArguments,
                TileOptions.ShowNameOnLogo | TileOptions.ShowNameOnWideLogo,
                logo);

            secondaryTile.ForegroundText = ForegroundText.Light;
            secondaryTile.SmallLogo = smallLogo;
            var buttonTransform = PinTileButton.TransformToVisual(null);
            var point = buttonTransform.TransformPoint(new Point());
            var rect = new Rect(point, new Size(PinTileButton.ActualWidth,
                PinTileButton.ActualHeight));
            bool isPinned = await secondaryTile.RequestCreateForSelectionAsync(
                rect, Windows.UI.Popups.Placement.Below);
            if (isPinned)
            {
                Feature6OutputText.Text = "Secondary tile successfully pinned.";
            }
            else
            {
                Feature6OutputText.Text = "Secondary tile not pinned.";
            }
        }

        private async void UnpinTileButtonTapped(object sender, TappedRoutedEventArgs e)
        {
            if (Windows.UI.StartScreen.SecondaryTile.Exists(SECONDARY_TILE_ID))
            {
                var secondaryTile = new SecondaryTile(SECONDARY_TILE_ID);
                var buttonTransform = UnpinTileButton.TransformToVisual(null);
                var point = buttonTransform.TransformPoint(new Point());
                var rect = new Rect(point, new Size(UnpinTileButton.ActualWidth,
                    UnpinTileButton.ActualHeight));
                bool isUnpinned = await secondaryTile.RequestDeleteForSelectionAsync(
                    rect, Windows.UI.Popups.Placement.Below);
                if (isUnpinned)
                {
                    Feature6OutputText.Text = "Secondary tile successfully unpinned.";
                }
                else
                {
                    Feature6OutputText.Text = "Secondary tile not unpinned.";
                }
            }
            else
            {
                Feature6OutputText.Text = "Secondary tile is not currently pinned.";
            }
        }

        public void StartedFromSecondaryTile(LaunchActivatedEventArgs args)
        {
            FeatureList.SelectedIndex = 6;
            Feature7OutputText.Text = "App strated with the following arguments: " 
                + args.Arguments;
        }
    }
}