﻿using System;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.Foundation;
using Windows.System;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace LaunchFiles
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        const string FILE = @"Assets\Surface.png";
        const string URI = "http://msdn.com";
        public MainPage()
        {
            this.InitializeComponent();
        }

        private async void StartWithDefaultButtonClick(object sender, RoutedEventArgs e)
        {
            string message;
            var file = await Package.Current.InstalledLocation.GetFileAsync(FILE);
            if (file != null)
            {
                bool success = await Launcher.LaunchFileAsync(file);
                message = success
                    ? "Das Bild wurde erfolgreich geladen."
                    : "Die Datei konnte nicht gestartet werden.";
            }
            else
            {
                message = "Die Datei wurde nicht gefunden.";
            }
            await DisplayMessage(message);
        }

        private async void StartWithWarningButtonClick(object sender, RoutedEventArgs e)
        {
            string message;
            var file = await Package.Current.InstalledLocation.GetFileAsync(FILE);
            if (file != null)
            {
                var options = new LauncherOptions();
                options.TreatAsUntrusted = true;
                bool success = await Launcher.LaunchFileAsync(file, options);
                message = success
                    ? "Das Bild wurde erfolgreich geladen."
                    : "Die Datei konnte nicht gestartet werden.";
            }
            else
            {
                message = "Die Datei wurde nicht gefunden.";
            }
            await DisplayMessage(message);
        }

        private async void StartOpenWithButtonClick(object sender, RoutedEventArgs e)
        {
            string message;
            var file = await Package.Current.InstalledLocation.GetFileAsync(FILE);
            if (file != null)
            {
                var transform = StartOpenWithButton.TransformToVisual(null);
                var popupLocation = transform.TransformPoint(new Point());
                popupLocation.Y += StartOpenWithButton.ActualHeight;
                var options = new LauncherOptions();
                options.DisplayApplicationPicker = true;
                options.UI.InvocationPoint = popupLocation;
                options.UI.PreferredPlacement = Placement.Below;
                bool success = await Launcher.LaunchFileAsync(file, options);
                message = success
                    ? "Das Bild wurde erfolgreich geladen."
                    : "Die Datei konnte nicht gestartet werden.";
            }
            else
            {
                message = "Die Datei wurde nicht gefunden.";
            }
            await DisplayMessage(message);
        }

        private async void StartUriWithDefaultButtonClick(object sender, RoutedEventArgs e)
        {
            string message;
            var file = await Package.Current.InstalledLocation.GetFileAsync(FILE);
            if (file != null)
            {
                bool success = await Launcher.LaunchUriAsync(new Uri(URI));
                message = success
                    ? "Der URI wurde erfolgreich aufgerufen."
                    : "Der URI konnte nicht aufgerufen werden.";
            }
            else
            {
                message = "Der URI wurde nicht gefunden.";
            }
            await DisplayMessage(message);
        }

        private async Task DisplayMessage(string message)
        {
            var dialog = new MessageDialog(message,
                "Dateistart");
            dialog.Commands.Add(new UICommand("OK"));
            await dialog.ShowAsync();
        }
    }
}
