﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using FamilyTrips.Common;
using Windows.Foundation.Metadata;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using System.Linq;

namespace FamilyTrips.Data
{
    /// <summary>
    /// This class represents the runtime data source of the application.
    /// </summary>
    public sealed class TripDataSource
    {
        private static TripDataSource _tripDataSource = new TripDataSource();

        /// <summary>
        /// Gets the container representing all trips
        /// </summary>
        public ObservableCollection<TripDataGroup> AllTrips { get; private set; }

        public static IEnumerable<TripDataGroup> GetTrips(string id)
        {
            if (!id.Equals("AllTrips"))
            {
                throw new ArgumentException("Only 'AllTrips' is supported as a collection of groups");
            }
            return _tripDataSource.AllTrips;
        }

        public static TripDataGroup GetTrip(string id)
        {
            var matches = _tripDataSource.AllTrips.Where((group) => group.Id.Equals(id));
            if (matches.Count() == 1) return matches.First();
            return null;
        }

        public static TripItem GetItem(string id)
        {
            // Simple linear search is acceptable for small data sets
            var matches = _tripDataSource.AllTrips
                .SelectMany(group => group.Items).Where((item) => item.Id.Equals(id));
            if (matches.Count() == 1) return matches.First();
            return null;
        }

        public TripDataSource()
        {
            String ITEM_CONTENT = String.Format("Item Content: {0}\n\n{0}\n\n{0}\n\n{0}\n\n{0}\n\n{0}\n\n{0}",
                        "Curabitur class aliquam vestibulum nam curae maecenas sed integer cras phasellus suspendisse quisque donec dis praesent accumsan bibendum pellentesque condimentum adipiscing etiam consequat vivamus dictumst aliquam duis convallis scelerisque est parturient ullamcorper aliquet fusce suspendisse nunc hac eleifend amet blandit facilisi condimentum commodo scelerisque faucibus aenean ullamcorper ante mauris dignissim consectetuer nullam lorem vestibulum habitant conubia elementum pellentesque morbi facilisis arcu sollicitudin diam cubilia aptent vestibulum auctor eget dapibus pellentesque inceptos leo egestas interdum nulla consectetuer suspendisse adipiscing pellentesque proin lobortis sollicitudin augue elit mus congue fermentum parturient fringilla euismod feugiat");

            AllTrips = new ObservableCollection<TripDataGroup>();

            var groupA = new TripDataGroup("Group-A",
                "A week in Amsterdam",
                "In the summer of 2012 we spent four days in Amsterdam. Originally I was there to attend at TechEd 2012 Europe, but I was happy my wife joined me for two days.",
                "Data/Amsterdam2012/Canal.jpg",
                "June, 2012");
            groupA.Items.Add(new TripItem("Group-A-1",
                "A horse-tracted cab",
                "When walking in the streets of Amsterdam, on a nice day we saw this cab.",
                "Data/Amsterdam2012/Cab.jpg",
                "Not an ordinary one...",
                ITEM_CONTENT,
                groupA));
            groupA.Items.Add(new TripItem("Group-A-2",
                "Seagull on a boat",
                "The first day we saw this seagull near to our accomodation.",
                "Data/Amsterdam2012/Seagull.JPG",
                "Is she the owner?",
                ITEM_CONTENT,
                groupA));
            groupA.Items.Add(new TripItem("Group-A-3",
                "Ducks in a park",
                "Near to the convention center, we walked in a large park full vith life.",
                "Data/Amsterdam2012/Ducks.JPG",
                "Birds everywhere...",
                ITEM_CONTENT,
                groupA));
            groupA.Items.Add(new TripItem("Group-A-4",
                "Herrengracht",
                "Everywhere you walk in Amsterdam, you cannot avoid the typical Dutch canals.",
                "Data/Amsterdam2012/Canal.JPG",
                "Water, water, water",
                ITEM_CONTENT,
                groupA));
            groupA.Items.Add(new TripItem("Group-A-5",
                "Windsor Rose",
                "Although tulips are said to be the typical Ducth flowers, parks are full with roses.",
                "Data/Amsterdam2012/Windsor rose.JPG",
                "Roses everywhere",
                ITEM_CONTENT,
                groupA));
            groupA.Items.Add(new TripItem("Group-A-6",
                "Nice Rose",
                "This is another rose, in contrary to the tulip.",
                "Data/Amsterdam2012/Rose.JPG",
                "Roses everywhere",
                ITEM_CONTENT,
                groupA));
            groupA.Items.Add(new TripItem("Group-A-7",
                "Amstel Station",
                "We're close to the Amstel station, accompanied with many bikes.",
                "Data/Amsterdam2012/Station.JPG",
                "Bikes everywhere",
                ITEM_CONTENT,
                groupA));
            AllTrips.Add(groupA);

            var groupR = new TripDataGroup("Group-R",
                "Sport in Riccione",
                "My wife is a master swimmer. In June, 2012 she attended at the World Masters Swimming Championship in Riccione, Italy. I joined her in the last three days.",
                "Data/Riccione2012/Dolphin Team.JPG",
                "June, 2012");
            groupR.Items.Add(new TripItem("Group-R-1",
                "Henriett",
                "My wife is waiting in the seeding area before the 200m butterfly.",
                "Data/Riccione2012/Henriett.JPG",
                "An Athletic wife",
                ITEM_CONTENT,
                groupR));
            groupR.Items.Add(new TripItem("Group-R-2",
                "Budapest Dolphins",
                "The four members of the Budapest Dolphin Swimming Team at the Masters Swimming World Championship.",
                "Data/Riccione2012/Dolphin Team.JPG",
                "A great swimming team",
                ITEM_CONTENT,
                groupR));
            groupR.Items.Add(new TripItem("Group-R-3",
                "Riccione Beach",
                "Attendees at the World Championship were welcomed by the organisers on the beach.",
                "Data/Riccione2012/Beach.JPG",
                "Hot welcome",
                ITEM_CONTENT,
                groupR));
            groupR.Items.Add(new TripItem("Group-R-4",
                "Bike Tour",
                "After the championship we visited the nice city of Ravenna. During our stay we took a bike tour to Dante's beach.",
                "Data/Riccione2012/Bike Tour.JPG",
                "A great day",
                ITEM_CONTENT,
                groupR));
            groupR.Items.Add(new TripItem("Group-R-5",
                "Fort of Ravenna",
                "At the end of the bike tour we visited the old fort of Ravenna.",
                "Data/Riccione2012/Fort.JPG",
                "Being very old",
                ITEM_CONTENT,
                groupR));
            groupR.Items.Add(new TripItem("Group-R-6",
                "Beautiful Temple",
                "Ravenna is full with temples, this is only one of them.",
                "Data/Riccione2012/Temple.JPG",
                "One of the many",
                ITEM_CONTENT,
                groupR));
            groupR.Items.Add(new TripItem("Group-R-7",
                "Museum of Modern Art",
                "We wanted to visit this great museum. Unfortunately, we arrived too late...",
                "Data/Riccione2012/Museum.JPG",
                "Missed inside",
                ITEM_CONTENT,
                groupR));
            AllTrips.Add(groupR);

            var groupMM = new TripDataGroup("Group-MM",
                "Mini Marathon Running",
                "My family is very athletic. In May, 2011 we attended at the Strucc Mini Marathon in Mosonmagyarovar, Hungary.",
                "Data/MiniMarathon2011/Family.jpg",
                "May, 2011");
            groupMM.Items.Add(new TripItem("Group-MM-1",
                "With My Daughters",
                "All of our family members like long-distance running. Here I am with my two daughters, just before warming up for the start.",
                "Data/MiniMarathon2011/Me With Daughters.JPG",
                "Before the start",
                ITEM_CONTENT,
                groupMM));
            groupMM.Items.Add(new TripItem("Group-MM-2",
                "Speed",
                "This run was 14km (9 miles). I'm just turning to the second lap of the two.",
                "Data/MiniMarathon2011/speed.JPG",
                "Reached the half distance",
                ITEM_CONTENT,
                groupMM));
            groupMM.Items.Add(new TripItem("Group-MM-3",
                "Eszter close to the Finnish",
                "My oldest daughter, Eszter is taking the last meters to the finnish -- with the help of my father-in-law.",
                "Data/MiniMarathon2011/Finnish.JPG",
                "She seems to be cheerful",
                ITEM_CONTENT,
                groupMM));
            groupMM.Items.Add(new TripItem("Group-MM-4",
                "Girls are tired",
                "Henriett and Eszter in their well-deserved relaxing after the run.",
                "Data/MiniMarathon2011/Tired.JPG",
                "Being thirsty",
                ITEM_CONTENT,
                groupMM));
            groupMM.Items.Add(new TripItem("Group-MM-5",
                "Novak Family",
                "Eszter, Henriett, Reka and me after the victory ceremony.",
                "Data/MiniMarathon2011/Family.JPG",
                "Having a successful competition",
                ITEM_CONTENT,
                groupMM));
            groupMM.Items.Add(new TripItem("Group-MM-6",
                "With my father-in-law",
                "Well, the best runner in the family is my father-in-law.",
                "Data/MiniMarathon2011/Medals.JPG",
                "The best runner",
                ITEM_CONTENT,
                groupMM));
            AllTrips.Add(groupMM);

            var groupC = new TripDataGroup("Group-C",
                "Cruising in the Caribbeans",
                "After the MVP Summit in 2010, I went to a Caribbena cruise with my wife to celebrate our 16th anniversary.",
                "Data/Caribbean2010/Valor.JPG",
                "March, 2010");
            groupC.Items.Add(new TripItem("Group-C-1",
                "Carnival Valor",
                "We took this huge cruising ship with about 3500 other passengers to the Caribbean.",
                "Data/Caribbean2010/Valor.JPG",
                "Great cruising ship",
                ITEM_CONTENT,
                groupC));
            groupC.Items.Add(new TripItem("Group-C-2",
                "Carpe diem",
                "I deserved relaxing after a great MVP Summit :-).",
                "Data/Caribbean2010/Istvan.JPG",
                "Seize the day",
                ITEM_CONTENT,
                groupC));
            groupC.Items.Add(new TripItem("Group-C-3",
                "Time to say good bye",
                "The Valor started from Miami. She slowly swimmed out to the sea.",
                "Data/Caribbean2010/Miami.JPG",
                "Starting from Miami",
                ITEM_CONTENT,
                groupC));
            groupC.Items.Add(new TripItem("Group-C-4",
                "Underwater in Honduras",
                "Although my scuba diving gear was not with me, I enjoyed snorkelling.",
                "Data/Caribbean2010/Underwater.JPG",
                "Snorkelling is fun...",
                ITEM_CONTENT,
                groupC));
            groupC.Items.Add(new TripItem("Group-C-5",
                "Henriett with a stringray",
                "Outr last sop was Stingray City at the Grand Cayman Islands. It was a fun being massaged by these tame beasts...",
                "Data/Caribbean2010/Henriett.JPG",
                "Stigray City",
                ITEM_CONTENT,
                groupC));
            groupC.Items.Add(new TripItem("Group-C-6",
                "Mermaid of the seas",
                "Not only me, but Henriett likes snorkelling, too.",
                "Data/Caribbean2010/Mermaid.JPG",
                "All of a sudden she was there...",
                ITEM_CONTENT,
                groupC));
            groupC.Items.Add(new TripItem("Group-C-7",
                "Water-lizard",
                "A not-so-small water-lizard we saw on the way back from Stingray City to the boat.",
                "Data/Caribbean2010/Varanus.JPG",
                "Hide-and-seek",
                ITEM_CONTENT,
                groupC));
            AllTrips.Add(groupC);

            var groupM = new TripDataGroup("Group-M",
                "Summer in Mauritius",
                "The year's end in 2008 was very tough, so we decided to relax in the wonderful island of Mauritius. Instead of staying in a hotel, we choose an apartement in an almost deserted coastline.",
                "Data/Mauritius2009/Apartement.jpg",
                "January, 2009");
            groupM.Items.Add(new TripItem("Group-M-1",
                "Excursion to a sanctuary",
                "One day we had an excursion near to a Hindi sanctuary where we saw this little altar.",
                "Data/Mauritius2009/Altar.JPG",
                "Peace and calm",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-2",
                "Cascade of charm",
                "Mauritius is full with waterfalls and cascades. We climbed up this one.",
                "Data/Mauritius2009/Cascade.JPG",
                "Is she the owner?",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-3",
                "Close to the shore",
                "Our apartement was very close to the shore. The shore was almost deserted, we could feel as we'd been alone.",
                "Data/Mauritius2009/Apartement.JPG",
                "Deserted",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-4",
                "Giant Turtle",
                "We visited several turtle farms while staying in Mauritius. In one of them we met with this old guy.",
                "Data/Mauritius2009/Giant Turtle.JPG",
                "MUst be old",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-5",
                "Together",
                "This morning started just like the others. We had our breakfast at the balcony of our apartement.",
                "Data/Mauritius2009/Together.JPG",
                "On the balcony of our apartement",
                 ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-6",
                "Among sculptures",
                "Mauritius is a place where the four great religions live together in harmony. One day we visited a Hindi sacred place.",
                "Data/Mauritius2009/Sanctuary.JPG",
                "One of sacred",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-7",
                "Nice hotel",
                "This hotel was about 20 minutes walk from our apartement. We often visited it for its pool.",
                "Data/Mauritius2009/Hotel.JPG",
                "Peaceful place to stay",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-8",
                "Lion Head",
                "From the shore we had a nice view to the Lion Head, at local people call this famous mountain.",
                "Data/Mauritius2009/Lion Head.JPG",
                "The famous mount of Mauritius",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-9",
                "Sea Urchin",
                "I took this photo during a snorkelling near to our apartement.",
                "Data/Mauritius2009/Sea urchin.JPG",
                "Close to the shore",
                ITEM_CONTENT,
                groupM));
            groupM.Items.Add(new TripItem("Group-M-10",
                "Tea grove",
                "Mauritius has famous tea farms. Everywhere in the country you can see tea groves and sugar-cane.",
                "Data/Mauritius2009/Tea grove.JPG",
                "Big green",
                ITEM_CONTENT,
                groupM));
            AllTrips.Add(groupM);

            var groupH = new TripDataGroup("Group-H",
                "Scuba Diving in Hurghada",
                "The Novak family is full with rabid fans of scuba diving. In the summer holiday of 2008 we visited Hurghada, Egypt with the kids for having fun.",
                "Data/Hurghada2008/With Kids.JPG",
                "July, 2008");
            groupH.Items.Add(new TripItem("Group-H-1",
                "Lionfish",
                "Would you mind that this beautiful fish is very poisonous? Its spikes are full with venom, if it stings you, you'd better pray for your life.",
                "Data/Hurghada2008/Lionfish.JPG",
                "Very poisonous",
                ITEM_CONTENT,
                groupH));
            groupH.Items.Add(new TripItem("Group-H-2",
                "Napoleon Fish",
                "This fish is bigger than one meter in its size, but very friendly, and curious. Swimms close to you to have time for peeking you.",
                "Data/Hurghada2008/Napoleon.JPG",
                "Friendly one",
                ITEM_CONTENT,
                groupH));
            groupH.Items.Add(new TripItem("Group-H-3",
                "With Kids",
                "Not only me, but also Eszter and Reka likes snorkeling. Here they were younger but enjoyed the nice underwater view of the Red Sea.",
                "Data/Hurghada2008/With Kids.JPG",
                "Eszter and Reka",
                ITEM_CONTENT,
                groupH));
            groupH.Items.Add(new TripItem("Group-H-4",
                "Scuba Family",
                "Eszter (12) was skilled enough to take a dive with us close to the reef of Gotta Abu Ramada.",
                "Data/Hurghada2008/Scuba Family.JPG",
                "Three under water",
                ITEM_CONTENT,
                groupH));
            groupH.Items.Add(new TripItem("Group-H-5",
                "Shoal of goat fishes",
                "Gotta Abu Ramada is full with vast number of goat fishes. You ca swimm into their shoals and feel you belong to them.",
                "Data/Hurghada2008/Shoal.JPG",
                "Great numbers",
                ITEM_CONTENT,
                groupH));
            groupH.Items.Add(new TripItem("Group-H-6",
                "Blue-spotted stingray",
                "One of the most common stingrays in the Red Sea. It is very shy and attacks you only if she does not have a chance to flee.",
                "Data/Hurghada2008/Stingray.JPG",
                "Tame and shy",
                ITEM_CONTENT,
                groupH));
            groupH.Items.Add(new TripItem("Group-H-7",
                "Hawkfish",
                "This colorful little fish tries to hide from you by staying motionless, so you can easily approach her to take a photo.",
                "Data/Hurghada2008/Hawkfish.JPG",
                "Motionless",
                ITEM_CONTENT,
                groupH));
            groupH.Items.Add(new TripItem("Group-H-7",
                "Triggerfish",
                "This big fish is always spying you, it seems like a nervous one. One I was attacked by one of them, he bit my fins.",
                "Data/Hurghada2008/Triggerfish.JPG",
                "Nervous",
                ITEM_CONTENT,
                groupH));
            AllTrips.Add(groupH);

            var groupSJ = new TripDataGroup("Group-SJ",
                "St. John's Diving Liveaboard",
                "Egypt is a fun place for scuba divers. In this trip I went to a scuba diving liveaboard close to the Sudanian borders of Egypt.",
                "Data/StJohns2008/Crocodile Fish.JPG",
                "May, 2008");
            groupSJ.Items.Add(new TripItem("Group-SJ-1",
                "Clownfish with her anemone",
                "Did you know that all clownfish are born as male? The dominant one changes sex and becomes female. If she dies, the next male changes its sex to take her place.",
                "Data/StJohns2008/Anemone.JPG",
                "Changing sex",
                ITEM_CONTENT,
                groupSJ));
            groupSJ.Items.Add(new TripItem("Group-SJ-2",
                "Clownfish with her anemone",
                "This lazy fish lies on the bottom all days. Its name is coming from its mouth, it opens like a crocodile's mouth.",
                "Data/StJohns2008/Crocodile Fish.JPG",
                "Lazy",
                ITEM_CONTENT,
                groupSJ));
            groupSJ.Items.Add(new TripItem("Group-SJ-3",
                "Moray eel",
                "This instance is a small one, but they often grow longer than 3 meters (10 feet). They seem to be wild, however they are shy. Often you can pet them.",
                "Data/StJohns2008/Moray Eel.JPG",
                "Blear-eyed",
                ITEM_CONTENT,
                groupSJ));
            groupSJ.Items.Add(new TripItem("Group-SJ-4",
                "Scallop",
                "These scallops grow large. If you are patient and slowly go close to them, you can see inside while they are open.",
                "Data/StJohns2008/Scallop.JPG",
                "Like a jewellery",
                ITEM_CONTENT,
                groupSJ));
            groupSJ.Items.Add(new TripItem("Group-SJ-5",
                "Napoleon Fish",
                "These great fish are used to divers. Everywhere our boat stopped, we could meet them.",
                "Data/StJohns2008/Napoleon.JPG",
                "Tame friends",
                ITEM_CONTENT,
                groupSJ));
            groupSJ.Items.Add(new TripItem("Group-SJ-6",
                "Turtle",
                "It is always great to swimm close to turtles. Unfortunately, ther are endangered, their number is less and less from year-by-year :-(",
                "Data/StJohns2008/Turtle.JPG",
                "Endangered",
                ITEM_CONTENT,
                groupSJ));
            AllTrips.Add(groupSJ);
        }
    }

    /// <summary>
    /// This abstract class represents a common base class for entities used in this app.
    /// </summary>
    [WebHostHidden]
    public abstract class TripDataCommon : BindableBase
    {
        private static Uri _baseUri = new Uri("ms-appx:///");

        private string _id = string.Empty;
        private string _title = string.Empty;
        private string _description = string.Empty;
        private ImageSource _image = null;
        private String _imagePath = null;

        public TripDataCommon(string id, string title, string description, string imagePath)
        {
            _id = id;
            _title = title;
            _description = description;
            _imagePath = imagePath;
        }

        public string Id
        {
            get { return _id; }
            set { SetProperty(ref _id, value); }
        }

        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }

        public string Description
        {
            get { return _description; }
            set { SetProperty(ref _description, value); }
        }

        public ImageSource Image
        {
            get
            {
                if (_image == null && _imagePath != null)
                {
                    _image = new BitmapImage(new Uri(_baseUri, _imagePath));
                }
                return _image;
            }

            set
            {
                _imagePath = null;
                SetProperty(ref _image, value);
            }
        }

        public void SetImage(String path)
        {
            _image = null;
            _imagePath = path;
            OnPropertyChanged("Image");
        }
    }

    public class TripDataGroup: TripDataCommon
    {
        private string _date;

        public TripDataGroup(string id, string title, string description, string imagePath, string date):
            base (id, title, description, imagePath)
        {
            _date = date;
            Items = new ObservableCollection<TripItem>();
        }

        public string Date
        {
            get { return _date; }
            set { SetProperty(ref _date, value);  }
        }

        public ObservableCollection<TripItem> Items { get; private set; }
    }

    public class TripItem : TripDataCommon
    {
        private string _subtitle;
        private string _content;
        private TripDataGroup _group;

        public TripItem(string id, string title, string description, string imagePath, string subtitle, string content, TripDataGroup group):
            base (id, title, description, imagePath)
        {
            _subtitle = subtitle;
            _content = content;
            _group = group;
        }

        public string Subtitle
        {
            get { return _subtitle; }
            set { SetProperty(ref _subtitle, value); }
        }

        public string Content
        {
            get { return _content; }
            set { SetProperty(ref _content, value); }
        }

        public TripDataGroup Group
        {
            get { return _group; }
            set { SetProperty(ref _group, value); }
        }
    }
}
