﻿using System;
using System.Collections.Generic;
using System.Linq;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using Windows.ApplicationModel.Contacts;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Provider;
using Windows.UI.Popups;

namespace PickerSample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        List<ContactInformation> contacts = null;

        public MainPage()
        {
            this.InitializeComponent();
        }

        async void OpenContacts(object sender, RoutedEventArgs e)
        {
            ContactPicker cp = new ContactPicker();
            var res = await cp.PickMultipleContactsAsync();
            if (res.Count > 0)
            {
                contacts = new List<ContactInformation>(res);
                lbxFiles.Items.Clear();
                foreach (var item in contacts) lbxFiles.Items.Add(item);
                tbkStatus.Text = string.Format("{0} Kontakt(e) ausgewählt", res.Count);
            }
        }

        async void SaveContactsToFile(object sender, RoutedEventArgs e)
        {
        if (contacts != null)
        {
            FileSavePicker fsp = new FileSavePicker { 
                SuggestedFileName = "contacts",
                DefaultFileExtension = ".txt" };
            fsp.SuggestedStartLocation = PickerLocationId.Desktop;
            fsp.FileTypeChoices.Add("Plain text", 
                new List<string>() {".txt" });
            StorageFile sf = await fsp.PickSaveFileAsync();

            if (sf != null)
            {
                CachedFileManager.DeferUpdates(sf);
                await FileIO.WriteLinesAsync(sf, 
                    contacts.Select(c => c.Name));
                if (await CachedFileManager.CompleteUpdatesAsync(sf) ==
                        FileUpdateStatus.Complete)
                    tbkStatus.Text = sf.Name + " wurde gespeichert.";
                else await new MessageDialog(
                    "Fehler beim Speichern der Datei",
                    "Error").ShowAsync();
            }
            else await new MessageDialog("Keine Datei ausgewählt.", 
                    "Error").ShowAsync();

        }
        else await new MessageDialog("Keine Kontakte ausgewählt.", "Error")
            .ShowAsync();

        }
    }
}
