﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using ContractsSample.Extenders;
using Windows.ApplicationModel.Search;

namespace ContractsSample
{
    public sealed partial class MainPage : Page
    {
        SearchPane sp = null;
	    List<NoteItem> notes = null;

        public MainPage()
        {
            this.InitializeComponent();
            ((App)Application.Current).Page = this;

            //Creating dummy data, and adding it to the ListBox
            notes = new List<NoteItem> {
                new NoteItem { Title = "Erinnerung", TakenAt = DateTime.Now.AddHours(-4), Text = "Hört nicht auf zu arbeiten. Spiele können warten." },
                new NoteItem { Title = "Wichtig!", TakenAt = DateTime.Now.AddHours(-8), Text = "Vergiss nicht zu essen." },
                new NoteItem { Title = "Äußerst wichtig!", TakenAt = DateTime.Now.AddHours(-7), Text = "Vergiss nicht, die Katze zu füttern." },
                new NoteItem { Title = "Neuer James-Bond-Film", TakenAt = DateTime.Now.AddHours(-8), Text = "Karten im Vorverkauf besorgen." },
                new NoteItem { Title = "Tief durchatmen", TakenAt = DateTime.Now.AddDays(-2), Text = "... sonst wirst du ohnmächtig." }
            };
            foreach (var n in notes) lbxNotes.Items.Add(n);

            sp = SearchPane.GetForCurrentView();
        }

        internal void ShowSearchResults(string searchText)
        {
            tbkFilter.Text = "Gefilterte Notizen mit " + searchText;
            lbxNotes.Items.Clear();
            foreach (var n in notes)
            {
                if (n.Title.ContainsSubstring(searchText))
                    lbxNotes.Items.Add(n);
            }
            btnRemoveFilter.IsEnabled = true;
        }

        void RemoveFilter(object sender, RoutedEventArgs e)
        {
            btnRemoveFilter.IsEnabled = false;
            tbkFilter.Text = string.Empty;
            lbxNotes.Items.Clear();
            foreach (var n in notes) lbxNotes.Items.Add(n);
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            sp.SuggestionsRequested += sp_SuggestionsRequested;
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);
            sp.SuggestionsRequested -= sp_SuggestionsRequested;
        }

        void sp_SuggestionsRequested(SearchPane sender,
            SearchPaneSuggestionsRequestedEventArgs args)
        {

            foreach (var s in
                lbxNotes.Items.OfType<NoteItem>().Select(n => n.Title))
            {
                if (s.ContainsSubstring(args.QueryText) &&
                    args.Request.SearchSuggestionCollection.Size < 5)
                    args.Request.SearchSuggestionCollection
                            .AppendQuerySuggestion(s);
            }
        }

        public async void AddNote(string title, DateTime createdAt,
                string text)
        {
            if (Dispatcher.HasThreadAccess)
            {
                notes.Add(new NoteItem
                {
                    Title = title,
                    TakenAt = createdAt,
                    Text = text
                });
                lbxNotes.Items.Clear();
                foreach (var n in notes) lbxNotes.Items.Add(n);
            }
            else
            {
                await Dispatcher.RunAsync(
                        Windows.UI.Core.CoreDispatcherPriority.High,
                        new Windows.UI.Core.DispatchedHandler(() =>
                        {
                            notes.Add(new NoteItem
                            {
                                Title = title,
                                TakenAt = createdAt,
                                Text = text
                            });
                            lbxNotes.Items.Clear();
                            foreach (var n in notes) lbxNotes.Items.Add(n);
                        }));
            }
        }


    }
}
