﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Networking.Connectivity;
using System.Text;
using Windows.Web.Syndication;
using Windows.UI.Popups;

namespace NetworkSample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            NetworkInformation.NetworkStatusChanged +=
                NetworkInformation_NetworkStatusChanged;
            RefreshNetworkInfo();
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            NetworkInformation.NetworkStatusChanged -=
                NetworkInformation_NetworkStatusChanged;
        }

        void NetworkInformation_NetworkStatusChanged(object sender)
        {
            RefreshNetworkInfo();
        }

        async void RefreshNetworkInfo()
        {
            ConnectionProfile profile =
                    NetworkInformation.GetInternetConnectionProfile();
            StringBuilder sb = new StringBuilder();

            if (profile != null)
            {
                sb.Append("Verbunden mit ");
                sb.Append(profile.GetNetworkNames()[0]);
                sb.Append(". Connectivity-Level ist ");
                sb.Append(profile.GetNetworkConnectivityLevel().ToString());
                sb.Append(".");
            }
            else sb.Append("Getrennt.");

            await Dispatcher.RunAsync(
                    Windows.UI.Core.CoreDispatcherPriority.Normal,
                    () => tbkNetStatus.Text = sb.ToString());
        }

        async void GetFeed(object sender, RoutedEventArgs e)
        {
            if (!string.IsNullOrWhiteSpace(tbFeedUrl.Text) &&
                Uri.IsWellFormedUriString(
                        tbFeedUrl.Text.Trim(), UriKind.Absolute))
            {
                SyndicationClient client = new SyndicationClient();
                try
                {
                    tbkFeedStatus.Text = "Feed herunterladen";
                    var feed = await client.RetrieveFeedAsync(
                            new Uri(tbFeedUrl.Text.Trim(),
                                    UriKind.Absolute));

                    tbkFeedStatus.Text = "Feed heruntergeladen";
                    lbxPosts.Items.Clear();
                    foreach (var item in feed.Items)
                        lbxPosts.Items.Add(item);
                }
                catch (Exception ex)
                {
                    tbkFeedStatus.Text = "Fehler: " + ex.Message;
                }
            }
        }

        void lbxPosts_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lbxPosts.SelectedIndex > -1)
            {
                SyndicationItem post =
                        lbxPosts.SelectedItem as SyndicationItem;
                tbkTitle.Text = post.Title.Text;
                tbkPublishedDate.Text = post.PublishedDate.ToString();
                wvSummary.NavigateToString(post.Summary.Text);
            }
        }

    }
}
