﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using LiveSample.Common;
using Microsoft.Live;
using Microsoft.Live.Controls;
using Windows.UI.Xaml.Media.Imaging;
using Windows.Storage.Pickers;
using Windows.Storage;

// Die Elementvorlage "Leere Seite" ist unter http://go.microsoft.com/fwlink/?LinkId=234238 dokumentiert.

namespace LiveSample
{
    /// <summary>
    /// Eine leere Seite, die eigenständig verwendet werden kann oder auf die innerhalb eines Rahmens navigiert werden kann.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        LiveConnectClient client = null;

        public MainPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Wird aufgerufen, wenn diese Seite in einem Rahmen angezeigt werden soll.
        /// </summary>
        /// <param name="e">Ereignisdaten, die beschreiben, wie diese Seite erreicht wurde. Die
        /// Parametereigenschaft wird normalerweise zum Konfigurieren der Seite verwendet.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }

        void SignInButton_SessionChanged(object sender,
            LiveConnectSessionChangedEventArgs e)
        {
            if (e.Status == LiveConnectSessionStatus.Connected)
            {
                client = new LiveConnectClient(e.Session);
                LoadProfile();
            }
            else if (e.Error != null) tbkStatus.Text = e.Error.Message;
        }

        async void LoadProfile()
        {
            LiveOperationResult lor = await client.GetAsync("me");
            tbkStatus.Text =
                "Willkommen, " + lor.Result["name"].ToString();
            lor = await client.GetAsync("me/picture");
            imgProfilePic.Source = new BitmapImage(new
                Uri(lor.Result["location"].ToString()));
        }

        async void btnGet_Click(object sender, RoutedEventArgs e)
        {
            if (client.Session != null)
            {
                LiveOperationResult lor =
                    await client.GetAsync(@"/me/skydrive/files");
                lbxContents.Items.Clear();
                foreach (var item in (dynamic)lor.Result["data"])
                {
                    lbxContents.Items.Add(new LiveItemInfo
                    {
                        Name = item.name,
                        Id = item.id,
                        Source = item.source
                    });
                }
            }
        }

        async void btnUploadFile_Click(object sender,
                RoutedEventArgs e)
        {
            var picker = new FileOpenPicker
            {
                ViewMode = PickerViewMode.List,
                SuggestedStartLocation = PickerLocationId.Desktop
            };
            picker.FileTypeFilter.Add(".jpg");
            picker.FileTypeFilter.Add(".jpeg");
            picker.FileTypeFilter.Add(".png");
            StorageFile file = await picker.PickSingleFileAsync();
            if (file != null)
            {
                tbkStatus.Text = "Bild hochladen...";
                try
                {
                    await client.BackgroundUploadAsync(
                        "me/skydrive", file.Name, file, 
                        OverwriteOption.Overwrite);
                    tbkStatus.Text = "Bild hochgeladen.";
                }
                catch (LiveConnectException ex)
                {
                    tbkStatus.Text = ex.Message;
                }
            }
        }


    }
}
