﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.Devices.Geolocation;
using Windows.UI.Core;
using System.Threading.Tasks;

namespace GeoLocationSample
{
    public sealed partial class MainPage : Page
    {
        Geolocator locator = null;
        
        public MainPage()
        {
            this.InitializeComponent();

            locator = new Geolocator(); 
            locator.StatusChanged += locator_StatusChanged;
        }

        async void locator_StatusChanged(Geolocator sender, StatusChangedEventArgs args)
        {
            await Dispatcher.RunAsync(CoreDispatcherPriority.Normal,
                new DispatchedHandler(() =>
                    tbkStatus.Text = "Geolocation-Dienst ist " +
                                         args.Status.ToString()
                ));
        }

        void tsLocationTracking_Toggled(object sender, RoutedEventArgs e)
        {
            if (tsLocationTracking.IsOn)
                locator.PositionChanged += locator_PositionChanged;
            else locator.PositionChanged -= locator_PositionChanged;
        }

        void tsAccuracy_Toggled(object sender, RoutedEventArgs e)
        {
            if (locator != null)
            {
                locator.DesiredAccuracy = tsAccuracy.IsOn ?
                    PositionAccuracy.High : PositionAccuracy.Default;
            }
        }

        async void locator_PositionChanged(Geolocator sender, PositionChangedEventArgs args)
        {
            await refreshUIAsync(args.Position);
        }

        async Task refreshUIAsync(Geoposition pos)
        {
            if (!Dispatcher.HasThreadAccess)
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal,
                    async () => await refreshUIAsync(pos));
            }
            else
            {
                tbkLat.Text = pos.Coordinate.Latitude.ToString();
                tbkLong.Text = pos.Coordinate.Longitude.ToString();
                tbkAlt.Text = pos.Coordinate.Altitude + " m";
                tbkAccuracy.Text = pos.Coordinate.Accuracy + " m";
                tbkCountry.Text = pos.CivicAddress.Country;
                tbkTimestamp.Text =
                    pos.Coordinate.Timestamp.ToString("T");
            }
        }

        async void RefreshData(object sender, RoutedEventArgs e)
        {
            if (tsLocationTracking.IsOn)
                await refreshUIAsync(await locator.GetGeopositionAsync());
        }
    }
}