﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Devices.Sensors;
using System;
using System.Threading.Tasks;
using Windows.UI.Popups;


// Die Elementvorlage "Leere Seite" ist unter http://go.microsoft.com/fwlink/?LinkId=234238 dokumentiert.

namespace AccelGyroSample
{
    /// <summary>
    /// Eine leere Seite, die eigenständig verwendet werden kann oder auf die innerhalb eines Rahmens navigiert werden kann.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        #region DependencyProperties
        public bool AcmEnabled
        {
            get { return (bool)GetValue(AcmEnabledProperty); }
            set { SetValue(AcmEnabledProperty, value); }
        }
        public static readonly DependencyProperty AcmEnabledProperty =
            DependencyProperty.Register("AcmEnabled", typeof(bool),
            typeof(MainPage), new PropertyMetadata(false));

        public bool GyroEnabled
        {
            get { return (bool)GetValue(GyroEnabledProperty); }
            set { SetValue(GyroEnabledProperty, value); }
        }
        public static readonly DependencyProperty GyroEnabledProperty =
            DependencyProperty.Register("GyroEnabled", typeof(bool),
            typeof(MainPage), new PropertyMetadata(false));

        #endregion

        Accelerometer acm = null;

        public MainPage()
        {
            this.InitializeComponent();
            acm = Accelerometer.GetDefault();
            if (acm != null) tbkAccelerometer.Text =
                "Beschleunigungsmesser bereit.";
            else tbkAccelerometer.Text =
                    "Beschleunigungsmesser nicht gefunden.";
        }

        async void btnEnableDisableAccelerometer_Click(object sender, RoutedEventArgs e)
        {
            if (acm != null)
            {
                if (!AcmEnabled)
                {
                    acm.ReadingChanged += acm_ReadingChanged;
                    AcmEnabled = true;
                    btnEnableDisableAccelerometer.Content =
                        "Beschleunigungsmesser deaktivieren";
                }
                else
                {
                    acm.ReadingChanged -= acm_ReadingChanged;
                    AcmEnabled = false;
                    btnEnableDisableAccelerometer.Content =
                        "Beschleunigungsmesser aktivieren";
                }
            }
            else
            {
                await new MessageDialog(
                    "Kein Beschleunigungsmesser vorhanden.",
                    "Fehler").ShowAsync();
            }
        }

        async void btnReadAccelerometer_Click(object sender,
                RoutedEventArgs e)
        {
            await Task.Delay(1000).ContinueWith(async t =>
            {
                var reading = acm.GetCurrentReading();
                await Dispatcher.RunAsync(
                        Windows.UI.Core.CoreDispatcherPriority.High,
                        () =>
                        {
                            tbkAccelerometer.Text = string.Format(
                                "X: {0}, Y: {1}, Z: {2} @ {3}:{4}",
                                reading.AccelerationX,
                                reading.AccelerationY,
                                reading.AccelerationZ,
                                reading.Timestamp.Minute,
                                reading.Timestamp.Second);
                        });
            });
        }

        async void acm_ReadingChanged(Accelerometer sender,
                AccelerometerReadingChangedEventArgs args)
        {
            await Dispatcher.RunAsync(
                    Windows.UI.Core.CoreDispatcherPriority.Normal,
                    () =>
                    {
                        var r = args.Reading;
                        rectAcm.Width = 100 + Math.Abs(50 * r.AccelerationX);
                        rectAcm.Height = 100 + Math.Abs(50 * r.AccelerationY);
                        byte rgb = (byte)Math.Max(
                                0, Math.Min(255, 128 + 10 * r.AccelerationZ));
                        rectAcm.Fill = new SolidColorBrush(
                                Color.FromArgb(255, rgb, rgb, rgb));
                    });
        }


    }
}
