#include "pch.h"
#include "Stopwatch.h"
#include "MatrixOperation.h"
#include <collection.h>
#include <ppltasks.h>
#include <amp.h>

using namespace CppDemo;

using namespace std;
using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Concurrency;

namespace MatrixDemo
{

// Initializes the matrix operation object
MatrixOperation::MatrixOperation()
{
}

// Starts the asynchron execution of sequential matrix multiplication
IAsyncOperation<unsigned long long>^ MatrixOperation::MultiplyMatrixSeriallyAsync()
{
	return create_async([]() -> unsigned long long
	{
		return DoMultiplication(&MatrixOperation::MultiplySequential);
	});
}

// Starts the asynchron execution of AMP matrix multiplication
IAsyncOperation<unsigned long long>^ MatrixOperation::MultiplyMatrixWithAMPAsync()
{
	return create_async([]() -> unsigned long long
	{
		return DoMultiplication(&MatrixOperation::MultiplyAMP);
	});
}

// Creates a random matrix to be the input of multiplication operations
void MatrixOperation::CreateRandomMatrix(int rows, int cols, vector<float>& matrix)
{
	srand((unsigned)time(0));
	matrix.erase(begin(matrix), end(matrix));
	for (int index = 0; index < rows*cols; index++)
	{
		matrix.push_back((float)(rand()/117*23.0));
	}
}

// This algorithm multiplies matrices sequentially
void MatrixOperation::MultiplySequential(vector<float>& vC, 
			const vector<float>& vA, 
			const vector<float>& vB, 
			int M, int N, int W)
{
}

// This algorithm multiplies matrices with AMP
void MatrixOperation::MultiplyAMP(vector<float>& vC, 
			const vector<float>& vA, 
			const vector<float>& vB, 
			int M, int N, int W)
{
}

// Carries out the full matrix multiplication including initialization
unsigned long long MatrixOperation::DoMultiplication(void (* mult)(vector<float>& vC, 
			const vector<float>& vA, 
			const vector<float>& vB, 
			int M, int N, int W))
{
	Stopwatch watch;
	const int MATRIX_SIZE = 400;
	vector<float> matrixA;
	vector<float> matrixB;
	vector<float> matrixC(MATRIX_SIZE*MATRIX_SIZE);

	CreateRandomMatrix(MATRIX_SIZE, MATRIX_SIZE, matrixA);
	CreateRandomMatrix(MATRIX_SIZE, MATRIX_SIZE, matrixB);
	watch.Start();
    (*mult)(matrixC, matrixA, matrixB, MATRIX_SIZE, MATRIX_SIZE, MATRIX_SIZE);
	return watch.Stop();
}

}