﻿#include "pch.h"
#include "MainPage.xaml.h"
#include "Stopwatch.h"
#include "MatrixOperation.h"
#include <collection.h>
#include <sstream>
#include <Windows.h>
#include <ppltasks.h>
#include <time.h>

using namespace CppDemo;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::System;
using namespace Windows::Storage;
using namespace std;
using namespace concurrency;

namespace WFC = Windows::Foundation::Collections;
namespace col = Platform::Collections;

// Initializes the page and sets the first demo to the active one.
MainPage::MainPage()
{
	InitializeComponent();

    FeatureList->SelectionChanged += ref new SelectionChangedEventHandler(this, &MainPage::ScenarioList_SelectionChanged);
    FeatureList->SelectedItem = Feature1;
}

// Initializes the selected feature and removes the previous output
void MainPage::ScenarioList_SelectionChanged(Object ^sender, SelectionChangedEventArgs ^e)
{
	ResetFeatures();
    ListBoxItem^ selectedDemo = dynamic_cast<ListBoxItem^>(FeatureList->SelectedItem);
	int selected = FeatureList->SelectedIndex;

    if (selected == 0)
    {
        Feature1Input->Visibility = Xaml::Visibility::Visible;
        Feature1Output->Visibility = Xaml::Visibility::Visible;
    }
    else if (selected == 1)
    {
        Feature2Input->Visibility = Xaml::Visibility::Visible;
        Feature2Output->Visibility = Xaml::Visibility::Visible;
    }
    else if (selected == 2)
    {
        Feature3Input->Visibility = Xaml::Visibility::Visible;
        Feature3Output->Visibility = Xaml::Visibility::Visible;
    }
    else if (selected == 3)
    {
        Feature4Input->Visibility = Xaml::Visibility::Visible;
        Feature4Output->Visibility = Xaml::Visibility::Visible;
    }
}

// Resets all features
void MainPage::ResetFeatures()
{
	Feature1Input->Visibility = Xaml::Visibility::Collapsed;
	Feature1Output->Visibility = Xaml::Visibility::Collapsed;
	Feature1OutputText->Text = "";

	Feature2Input->Visibility = Xaml::Visibility::Collapsed;
	Feature2Output->Visibility = Xaml::Visibility::Collapsed;
	Feature2OutputText->Text = "";

	Feature3Input->Visibility = Xaml::Visibility::Collapsed;
	Feature3Output->Visibility = Xaml::Visibility::Collapsed;
	Feature3OutputText->Text = "";

	Feature4Input->Visibility = Xaml::Visibility::Collapsed;
	Feature4Output->Visibility = Xaml::Visibility::Collapsed;
	Feature4OutputText->Text = "";
}

// Feature #1: "Reverse" button event handler 
void MainPage::ReverseStringButton_Tapped(Object^ sender, TappedRoutedEventArgs^ e)
{
    String^ yourName = YourName->Text;
    Feature1OutputText->Text += 
            "Ihr Name: " + yourName + "\n";
    wstring inputString(yourName->Data());
    wstring reverseString(inputString.rbegin(), 
            inputString.rend());
    String^ yourReversedName = 
            ref new String(reverseString.c_str());
    Feature1OutputText->Text += 
            "Umgekehrt: " + yourReversedName + "\n";
}

// Feature #2: Create a random vector
col::Vector<int>^ MainPage::CreateRandomVector()
{
    vector<int> myNumbers;
    srand((unsigned)time(0));
    for (int i = 0; i < 10; i++)
    {
        myNumbers.push_back(rand()%100 + 1);
    }
    return ref new col::Vector<int>(move(myNumbers));
}

// Feature #2: "Sum Up a Vector" button event handler
void MainPage::SumAVectorButton_Tapped(Object^ sender, TappedRoutedEventArgs^ e)
{
    wstringstream streamVal;
    WFC::IVectorView<int>^ vv = 
            CreateRandomVector()->GetView();
    int sum = 0;
    int index = 0;
    for_each(begin(vv), end(vv), [&](int value)
    {
        streamVal << "Number " << ++index << " is " << 
                value << endl;
        sum += value;
    });
    streamVal << "Die Summe dieser Werte ist " << sum;
    Feature2OutputText->Text = 
            ref new String(streamVal.str().c_str());
}

// Feature #3: "Write Message to File" button event handler
void MainPage::WriteFileButton_Tapped(Object^ sender, TappedRoutedEventArgs^ e)
{
    Feature3OutputText->Text += 
            "Erstellung der Datei wurde gestartet.\n";
    StorageFolder^ documentsFolder = 
            KnownFolders::DocumentsLibrary;

    // --- Erstellung der Datei beginnen
    task<StorageFile^> createTask(
            documentsFolder->CreateFileAsync(
               "MySample.txt", 
               CreationCollisionOption::ReplaceExisting));
    createTask.then([this](StorageFile^ file)
    {
        Feature3OutputText->Text += 
                "Datei " + file->Name + " wurde erstellt.\n";
        // --- Schreiben in die Datei starten
        task<void> writeTask(FileIO::WriteTextAsync(
                file, YourMessage->Text));
        writeTask.then([this](void)
        {
            Feature3OutputText->Text += 
                    "Der Text '" + YourMessage->Text +
                    "' wurde in die Datei geschrieben.\n";
            });
        });
}

// Feature #4: "Multiply Matrices Sequentially" button event handler
void MainPage::CalculateSerialButton_Tapped(Object^ sender, TappedRoutedEventArgs^ e)
{
}

// Feature #4: "Multiply Matrices with AMP" button event handler
void MainPage::CalculateWithAMPButton_Tapped(Object^ sender, TappedRoutedEventArgs^ e)
{
}
