﻿using System;
using System.Diagnostics;
using Windows.ApplicationModel.Background;
using Windows.Storage;
using Windows.System.Threading;

namespace InternetStatusTask
{
    public sealed class StatusWatcherTask : IBackgroundTask
    {
        private volatile bool _cancelRequested = false;
        private BackgroundTaskDeferral _taskDeferral = null;
        uint _progress = 0;

        ThreadPoolTimer _timer = null;
        IBackgroundTaskInstance _instance;

        public void Run(IBackgroundTaskInstance taskInstance)
        {
            taskInstance.Canceled +=
                new BackgroundTaskCanceledEventHandler(OnCanceled);
            _taskDeferral = taskInstance.GetDeferral();
            _instance = taskInstance;
            _timer = ThreadPoolTimer.CreatePeriodicTimer(
                new TimerElapsedHandler(TimerCallback),
                TimeSpan.FromMilliseconds(2000));
        }

        private void OnCanceled(IBackgroundTaskInstance sender,
            BackgroundTaskCancellationReason reason)
        {
            _cancelRequested = true;
        }

        private void TimerCallback(ThreadPoolTimer timer)
        {
            if (_cancelRequested == false && _progress < 100)
            {
                _progress += 10;
                _instance.Progress = _progress;
            }
            else
            {
                _timer.Cancel();
                var settings =
                    ApplicationData.Current.LocalSettings;
                var key = _instance.Task.TaskId.ToString();
                settings.Values[key] = _cancelRequested
                    ? "Aufgabe abgebrochen."
                    : "Aufgabe abgeschlossen.";
                _taskDeferral.Complete();
            }
        }
    }
}
