﻿using System;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;
using Windows.ApplicationModel.Background;
using Windows.Storage;

namespace InternetStatusTaskSample
{
    public sealed partial class MainPage : Page
    {
        private CoreDispatcher StatusDispatcher;

        public MainPage()
        {
            this.InitializeComponent();
            StatusDispatcher =
                    Window.Current.CoreWindow.Dispatcher;
            UnregisterTaskButton.IsEnabled = false;
        }

        private void RegisterTask_Tapped(object sender, RoutedEventArgs e)
        {
            var builder = new BackgroundTaskBuilder();
            builder.Name = "StatusWatcherTask";
            builder.TaskEntryPoint =
                    "InternetStatusTask.StatusWatcherTask";
            builder.SetTrigger(new SystemTrigger(
                SystemTriggerType.NetworkStateChange, false));
            IBackgroundTaskRegistration task =
                    builder.Register();
            task.Completed += OnCompleted;
            RegisterTaskButton.IsEnabled = false;
            UnregisterTaskButton.IsEnabled = true;
            OutputText.Text +=
                    "StatusWatcherTask wurde registriert.\n";
        }

        private void UnregisterTask_Tapped(object sender, TappedRoutedEventArgs e)
        {
            foreach (var cur in BackgroundTaskRegistration.AllTasks)
            {
                if (cur.Value.Name == "StatusWatcherTask")
                {
                    cur.Value.Unregister(true);
                    OutputText.Text +=
                        "Die Registrierung von StatusWatcherTask ";
                    OutputText.Text += "wurde aufgehoben.\n";
                }
            }
            RegisterTaskButton.IsEnabled = true;
            UnregisterTaskButton.IsEnabled = false;
        }

        private async void OnCompleted(
                IBackgroundTaskRegistration task,
                BackgroundTaskCompletedEventArgs args)
        {
            await StatusDispatcher.RunAsync(
                CoreDispatcherPriority.Normal,
                () =>
                {
                    try
                    {
                        var key = task.TaskId.ToString();
                        var settings = 
                            ApplicationData.Current.LocalSettings;
                        OutputText.Text += 
                            settings.Values[key].ToString() + "\n";
                    }
                    catch (Exception ex)
                    {
                        OutputText.Text += ex.ToString() + "\n";
                    }
                });
         }
    }
}
