﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Windows.Devices.Input;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace InputDevices
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        private void ListCapabilities_Tapped(object sender, TappedRoutedEventArgs e)
        {
            var output = new StringBuilder();

            output.Append("--- Fingereingabefunktionen abfragen\n");
            var touchCaps = new TouchCapabilities();
            output.Append(touchCaps.TouchPresent == 0
                ? "Es ist kein Fingereingabegerät angeschlossen.\n"
                : "Es ist ein Fingereingabegerät angeschlossen.\n");
            output.AppendFormat(
                "Das Fingereingabegerät unterstützt {0} Kontakte.\n",
                touchCaps.Contacts);

            output.Append("\n--- Tastaturfunktionen abfragen\n");
            var keyboardCaps = new KeyboardCapabilities();
            output.Append(keyboardCaps.KeyboardPresent == 0
                ? "Es ist keine Tastatur angeschlossen.\n"
                : "Es ist eine Tastatur angeschlossen.\n");

            output.Append("\n--- Mausfunktionen abfragen\n");
            var mouseCaps = new MouseCapabilities();
            output.Append(mouseCaps.MousePresent == 0
                ? "Es ist keine Maus angeschlossen.\n"
                : "Es ist eine Maus angeschlossen.\n");
            output.AppendFormat("Die Maus hat {0} Tasten.\n", mouseCaps.NumberOfButtons);
            output.AppendFormat(
                "Der Benutzer hat die Maustasten {0}vertauscht.\n",
                mouseCaps.SwapButtons == 0 ? "nicht " : "");
            output.Append(mouseCaps.VerticalWheelPresent == 0
                ? "Es gibt kein vertikales Mausrad.\n"
                : "Es gibt ein vertikales Mausrad.\n");
            output.Append(mouseCaps.HorizontalWheelPresent == 0
                ? "Es gibt kein horizontales Mausrad.\n"
                : "Es gibt ein horizontales Mausrad.\n");

            output.Append("\n--- Zeigegerätinformationen abfragen\n");
            var index = 0;
            foreach (var device in PointerDevice.GetPointerDevices())
            {
                string deviceType;
                switch (device.PointerDeviceType)
                {
                    case PointerDeviceType.Mouse:
                        deviceType = "Maus-Gerät";
                        break;
                    case PointerDeviceType.Pen:
                        deviceType = "Eingabestift-Gerät";
                        break;
                    case PointerDeviceType.Touch:
                        deviceType = "Fingereingabe-Gerät";
                        break;
                    default:
                        deviceType = "unbekanntes Gerät";
                        break;
                }
                output.AppendFormat("Gerät #{0} ist ein {1} {2}"
                    + " mit {3} Kontakten.\n",
                    index,
                    device.IsIntegrated ? "internes" : "externes",
                    deviceType,
                    device.MaxContacts);
                var rect = device.PhysicalDeviceRect;
                output.AppendFormat("Gerät #{0} unterstützt " +
                    " das [{1},{2}]-[{3},{4}]-Rechteck.\n",
                    index,
                    rect.Left,
                    rect.Top,
                    rect.Right,
                    rect.Bottom);
                index++;
            }

            OutputText.Text = output.ToString();
        }
    }
}
