﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Store;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace LicenseDemo
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            //CurrentAppSimulator mit Testdaten konfigurieren
            var proxyDataFolder = await
                Package.Current.InstalledLocation
                    .GetFolderAsync("data");
            var proxyFile = await proxyDataFolder
                    .GetFileAsync("trial.xml");
            await CurrentAppSimulator.ReloadSimulatorAsync(proxyFile);

            //Lizenzinformationen abrufen
            var licenseInfo = CurrentAppSimulator.LicenseInformation;

            //Die UI aktualisieren
            RefreshLicenseDisplayInfo(licenseInfo);

            //Lizenzänderungen abonnieren
            licenseInfo.LicenseChanged += () =>
            {
                Dispatcher.RunAsync(
                        Windows.UI.Core.CoreDispatcherPriority.Normal,
                        () =>
                        {
                            RefreshLicenseDisplayInfo(licenseInfo);
                        });
            };

        }

        private void RefreshLicenseDisplayInfo(LicenseInformation licenseInfo)
        {

            if (licenseInfo.IsActive && licenseInfo.IsTrial)
            {   // Testversion
                txtLicenseStatus.Text = "Testversion läuft ab am " +
                    CurrentAppSimulator.LicenseInformation.ExpirationDate;
                btnBuyApp.Visibility = Visibility.Visible;
            }
            else if (licenseInfo.IsActive && !licenseInfo.IsTrial)
            {   // Volllizenz
                txtLicenseStatus.Text = "Vollfunktionsfähige App";
                btnBuyApp.Visibility = Visibility.Collapsed;
            }
            else
            {   // Fehler, oder die Testversion ist abgelaufen
                txtLicenseStatus.Text = "Testversion abgelaufen";
                btnBuyApp.Visibility = Visibility.Visible;
            }
        }

        private async void btnBuyApp_Click_1(object sender,
                RoutedEventArgs e)
        {
            await CurrentAppSimulator.RequestAppPurchaseAsync(false);
        }

    }
}
